@@ -1,7 +1,6 @@
 #ifndef GNUTLS_MPI_H
 # define GNUTLS_MPI_H
 
-# include <gnutls_int.h>
 # include <gcrypt.h>
 # include <libtasn1.h>
 
@@ -26,10 +25,7 @@
 #define _gnutls_mpi_mul gcry_mpi_mul
 #define _gnutls_mpi_add gcry_mpi_add
 #define _gnutls_mpi_add_ui gcry_mpi_add_ui
-#define _gnutls_mpi_sub_ui gcry_mpi_sub_ui
 #define _gnutls_mpi_mul_ui gcry_mpi_mul_ui
-#define _gnutls_prime_check gcry_prime_check
-#define _gnutls_mpi_div gcry_mpi_div
 
 # define _gnutls_mpi_alloc_like(x) _gnutls_mpi_new(_gnutls_mpi_get_nbits(x)) 
 # define _gnutls_mpi_salloc_like(x) _gnutls_mpi_snew(_gnutls_mpi_get_nbits(x)) 
@@ -42,7 +38,5 @@ int _gnutls_mpi_scan_pgp( GNUTLS_MPI *ret_mpi, const opaque *buffer, size_t *nby
 int _gnutls_mpi_print( void *buffer, size_t *nbytes, const GNUTLS_MPI a );
 int _gnutls_mpi_print_lz( void *buffer, size_t *nbytes, const GNUTLS_MPI a );
 
-int _gnutls_mpi_dprint_lz( gnutls_datum* dest, const GNUTLS_MPI a );
-#define _gnutls_mpi_dprint _gnutls_mpi_dprint_lz
 
 #endif
