@@ -8,16 +8,6 @@ typedef enum gnutls_x509_subject_alt_name {
 
 # ifdef LIBGNUTLS_VERSION	/* These are defined only in gnutls.h */
 
-/* Callback prototypes for the certificate authentication
- * callbacks.
- */
-typedef int gnutls_certificate_client_select_function(gnutls_session,
-      const gnutls_datum * client_cert, int ncerts,
-      const gnutls_datum * req_ca_cert, int nreqs);
-typedef int gnutls_certificate_server_select_function(gnutls_session,
-      const gnutls_datum * server_certs, int ncerts);
-
-
 struct gnutls_openpgp_key_int;
 typedef struct gnutls_openpgp_key_int *gnutls_openpgp_key;
 
@@ -65,11 +55,6 @@ int gnutls_rsa_export_get_modulus_bits(gnutls_session session);
 
 /* X509PKI */
 
-void gnutls_certificate_client_set_select_function(gnutls_session,
-	gnutls_certificate_client_select_function *);
-void gnutls_certificate_server_set_select_function(gnutls_session,
-	gnutls_certificate_server_select_function *);
-
 /* These are set on the credentials structure.
  */
 void gnutls_certificate_client_set_retrieve_function(gnutls_certificate_client_credentials,
@@ -129,6 +114,24 @@ int gnutls_pem_base64_decode_alloc(const char *header,
 #define GNUTLS_KEY_ENCIPHER_ONLY                1
 #define GNUTLS_KEY_DECIPHER_ONLY                32768
 
+typedef struct gnutls_params_st {
+	gnutls_params_type type;
+	union params {
+		gnutls_dh_params dh;
+		gnutls_rsa_params rsa_export;
+	} params;
+	int deinit;
+} gnutls_params_st;
+
+typedef int gnutls_params_function(gnutls_session, gnutls_params_type,
+	gnutls_params_st*);
+
+void gnutls_certificate_set_params_function(gnutls_certificate_credentials res, 
+	gnutls_params_function* func);
+void gnutls_anon_set_params_function(gnutls_certificate_credentials res, 
+	gnutls_params_function* func);
+
+
 
 # endif				/* LIBGNUTLS_VERSION */
 
