@@ -50,7 +50,7 @@ int _gnutls_x509_crt_get_extension( gnutls_x509_crt cert, const char* extension_
 	char str_critical[10];
 	int critical = 0;
 	char extnID[128];
-	char extnValue[256];
+	gnutls_datum value;
 	int indx_counter = 0;
 
 	ret->data = NULL;
@@ -91,60 +91,44 @@ int _gnutls_x509_crt_get_extension( gnutls_x509_crt cert, const char* extension_
 				return _gnutls_asn2err(result);
 			}
 
-			_gnutls_str_cpy(name2, sizeof(name2), name);
-			_gnutls_str_cat(name2, sizeof(name2), ".critical"); 
+			/* Handle Extension 
+			 */
+			if ( strcmp(extnID, extension_id)==0 && indx == indx_counter++) { 
+				/* extension was found 
+				 */
 
-			len = sizeof(str_critical);
-			result =
-			    asn1_read_value(cert->cert, name2, str_critical, &len);
-
-			if (result == ASN1_ELEMENT_NOT_FOUND) {
-				gnutls_assert();
-				break;
-			} else if (result != ASN1_SUCCESS) {
-				gnutls_assert();
-				return _gnutls_asn2err(result);
-			}
-
-			if (str_critical[0] == 'T')
-				critical = 1;
-			else critical = 0;
-
-			_gnutls_str_cpy(name2, sizeof(name2), name);
-			_gnutls_str_cat(name2, sizeof(name2), ".extnValue"); 
-
-			len = sizeof(extnValue) - 1;
-			result =
-			    asn1_read_value(cert->cert, name2, extnValue, &len);
-
-			if (result == ASN1_ELEMENT_NOT_FOUND)
-				break;
-			else {
-				if (result == ASN1_MEM_ERROR
-				    && critical == 0) {
-
-					_gnutls_x509_log
-					    ("X509_EXT: Cannot parse extension: %s. Too small buffer.",
-					     extnID);
-
-					continue;
-				}
-				if (result != ASN1_SUCCESS) {
+				/* read the critical status.
+				 */
+				len = sizeof(str_critical);
+				result =
+				    asn1_read_value(cert->cert, name2, str_critical, &len);
+	
+				if (result == ASN1_ELEMENT_NOT_FOUND) {
+					gnutls_assert();
+					break;
+				} else if (result != ASN1_SUCCESS) {
 					gnutls_assert();
 					return _gnutls_asn2err(result);
 				}
-			}
 
-			/* Handle Extension */
-			if ( strcmp(extnID, extension_id)==0 && indx == indx_counter++) { 
-				/* extension was found */
+				if (str_critical[0] == 'T')
+					critical = 1;
+				else critical = 0;
 
-				ret->data = gnutls_malloc( len);
-				if (ret->data==NULL)
-					return GNUTLS_E_MEMORY_ERROR;	
+				/* read the value.
+				 */
+				_gnutls_str_cpy(name2, sizeof(name2), name);
+				_gnutls_str_cat(name2, sizeof(name2), ".extnValue"); 
 
-				ret->size = len;
-				memcpy( ret->data, extnValue, len);
+				result = _gnutls_x509_read_value( cert->cert, name2, 
+					&value, 0);
+				if (result < 0) {
+					gnutls_assert();
+					return result;
+				}
+				
+				ret->data = value.data;
+				ret->size = value.size;
 				
 				if (_critical)
 					*_critical = critical;
@@ -176,8 +160,6 @@ int _gnutls_x509_crt_get_extension_oid( gnutls_x509_crt cert,
 	int k, result, len;
 	char name[128], name2[128], counter[MAX_INT_DIGITS];
 	char str[1024];
-	char str_critical[10];
-	int critical = 0;
 	char extnID[128];
 	char extnValue[256];
 	int indx_counter = 0;
@@ -217,51 +199,8 @@ int _gnutls_x509_crt_get_extension_oid( gnutls_x509_crt cert,
 				return _gnutls_asn2err(result);
 			}
 
-			_gnutls_str_cpy(name2, sizeof(name2), name);
-			_gnutls_str_cat(name2, sizeof(name2), ".critical"); 
-
-			len = sizeof(str_critical);
-			result =
-			    asn1_read_value(cert->cert, name2, str_critical, &len);
-
-			if (result == ASN1_ELEMENT_NOT_FOUND) {
-				gnutls_assert();
-				break;
-			} else if (result != ASN1_SUCCESS) {
-				gnutls_assert();
-				return _gnutls_asn2err(result);
-			}
-
-			if (strcmp( str_critical, "TRUE")==0)
-				critical = 1;
-			else critical = 0;
-
-			_gnutls_str_cpy(name2, sizeof(name2), name);
-			_gnutls_str_cat(name2, sizeof(name2), ".extnValue"); 
-
-			len = sizeof(extnValue) - 1;
-			result =
-			    asn1_read_value(cert->cert, name2, extnValue, &len);
-
-			if (result == ASN1_ELEMENT_NOT_FOUND)
-				break;
-			else {
-				if (result == ASN1_MEM_ERROR
-				    && critical == 0) {
-
-					_gnutls_x509_log
-					    ("X509_EXT: Cannot parse extension: %s. Too small buffer.",
-					     extnID);
-
-					continue;
-				}
-				if (result != ASN1_SUCCESS) {
-					gnutls_assert();
-					return _gnutls_asn2err(result);
-				}
-			}
-
-			/* Handle Extension */
+			/* Handle Extension 
+			 */
 			if ( indx == indx_counter++) { 
 				len = strlen( extnID) + 1;
 
@@ -294,7 +233,7 @@ int _gnutls_x509_crt_get_extension_oid( gnutls_x509_crt cert,
  *
  * Critical will be either 0 or 1.
  */
-int _gnutls_x509_crt_set_extension( gnutls_x509_crt cert, const char* extension_id, 
+static int set_extension( ASN1_TYPE asn, const char* extension_id, 
 	const gnutls_datum* ext_data, unsigned int critical)
 {
 	int result;
@@ -302,13 +241,13 @@ int _gnutls_x509_crt_set_extension( gnutls_x509_crt cert, const char* extension_
 
 	/* Add a new extension in the list.
 	 */
-	result = asn1_write_value(cert->cert, "tbsCertificate.extensions", "NEW", 1);
+	result = asn1_write_value(asn, "tbsCertificate.extensions", "NEW", 1);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		return _gnutls_asn2err(result);
 	}
 
-	result = asn1_write_value(cert->cert, "tbsCertificate.extensions.?LAST.extnID", extension_id, 1);
+	result = asn1_write_value(asn, "tbsCertificate.extensions.?LAST.extnID", extension_id, 1);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		return _gnutls_asn2err(result);
@@ -318,13 +257,13 @@ int _gnutls_x509_crt_set_extension( gnutls_x509_crt cert, const char* extension_
 	else str = "TRUE";
 	
 
-	result = asn1_write_value(cert->cert, "tbsCertificate.extensions.?LAST.critical", str, 1);
+	result = asn1_write_value(asn, "tbsCertificate.extensions.?LAST.critical", str, 1);
 	if (result != ASN1_SUCCESS) {
 		gnutls_assert();
 		return _gnutls_asn2err(result);
 	}
 
-	result = _gnutls_x509_write_value( cert->cert, "tbsCertificate.extensions.?LAST.extnValue",
+	result = _gnutls_x509_write_value( asn, "tbsCertificate.extensions.?LAST.extnValue",
 		ext_data, 0);
 	if (result < 0) {
 		gnutls_assert();
@@ -334,6 +273,119 @@ int _gnutls_x509_crt_set_extension( gnutls_x509_crt cert, const char* extension_
 	return 0;
 }
 
+/* Overwrite the given extension (using the index)
+ * index here starts from one.
+ */
+static int overwrite_extension( ASN1_TYPE asn, unsigned int indx,
+	const gnutls_datum *ext_data, unsigned int critical)
+{
+char name[128], name2[128], counter[MAX_INT_DIGITS];
+const char* str;
+int result;
+
+	_gnutls_str_cpy(name, sizeof(name), "tbsCertificate.extensions.?"); 
+	_gnutls_int2str(indx, counter); 
+	_gnutls_str_cat(name, sizeof(name), counter); 
+
+	if (critical==0) str = "FALSE";
+	else str = "TRUE";
+
+	_gnutls_str_cpy(name2, sizeof(name2), name);
+	_gnutls_str_cat(name2, sizeof(name2), ".critical"); 
+
+	result = asn1_write_value(asn, name2, str, 1);
+	if (result != ASN1_SUCCESS) {
+		gnutls_assert();
+		return _gnutls_asn2err(result);
+	}
+
+	_gnutls_str_cpy(name2, sizeof(name2), name);
+	_gnutls_str_cat(name2, sizeof(name2), ".extnValue"); 
+
+	result = _gnutls_x509_write_value( asn, name2, ext_data, 0);
+	if (result < 0) {
+		gnutls_assert();
+		return result;
+	}
+}
+
+/* This function will attempt to overwrite the requested extension with
+ * the given one. 
+ *
+ * Critical will be either 0 or 1.
+ */
+int _gnutls_x509_crt_set_extension( gnutls_x509_crt cert, const char* ext_id,
+	const gnutls_datum* ext_data, unsigned int critical)
+{
+	int result;
+	const char *str;
+	int k, len;
+	char name[128], name2[128], counter[MAX_INT_DIGITS];
+	char str_critical[10];
+	char extnID[128];
+	gnutls_datum value;
+	int indx_counter = 0;
+
+	/* Find the index of the given extension.
+	 */
+	k = 0;
+	do {
+		k++;
+
+		_gnutls_str_cpy(name, sizeof(name), "tbsCertificate.extensions.?"); 
+		_gnutls_int2str(k, counter); 
+		_gnutls_str_cat(name, sizeof(name), counter); 
+
+		len = sizeof(extnID) - 1;
+		result = asn1_read_value(cert->cert, name, extnID, &len);
+
+		/* move to next
+		 */
+
+		if (result == ASN1_ELEMENT_NOT_FOUND) {
+			break;
+		}
+
+		do {
+
+			_gnutls_str_cpy(name2, sizeof(name2), name);
+			_gnutls_str_cat(name2, sizeof(name2), ".extnID"); 
+
+			len = sizeof(extnID) - 1;
+			result =
+			    asn1_read_value(cert->cert, name2, extnID, &len);
+
+			if (result == ASN1_ELEMENT_NOT_FOUND) {
+				gnutls_assert();
+				break;
+			} else if (result != ASN1_SUCCESS) {
+				gnutls_assert();
+				return _gnutls_asn2err(result);
+			}
+
+			/* Handle Extension 
+			 */
+			if ( strcmp(extnID, ext_id)==0) { 
+				/* extension was found 
+				 */
+				return overwrite_extension( cert->cert, k, ext_data, critical);
+			}
+
+
+		} while (0);
+	} while (1);
+
+	if (result == ASN1_ELEMENT_NOT_FOUND) {
+		return set_extension( cert->cert, ext_id, ext_data, critical);
+	} else {
+		gnutls_assert();
+		return _gnutls_asn2err(result);
+	}
+
+	
+	return 0;
+}
+
 
 /* Here we only extract the KeyUsage field, from the DER encoded
  * extension.
@@ -623,6 +675,42 @@ int _gnutls_x509_ext_gen_key_id(const void* id, size_t id_size, gnutls_datum* de
 	return 0;
 }
 
+/* generate the AuthorityKeyID in a DER encoded extension
+ */
+int _gnutls_x509_ext_gen_auth_key_id(const void* id, size_t id_size, gnutls_datum* der_ext)
+{
+	ASN1_TYPE ext = ASN1_TYPE_EMPTY;
+	int result;
+
+	result = asn1_create_element(_gnutls_get_pkix(), "PKIX1.AuthorityKeyIdentifier", &ext);
+	if (result != ASN1_SUCCESS) {
+		gnutls_assert();
+		return _gnutls_asn2err(result);
+	}
+
+	result = asn1_write_value(ext, "keyIdentifier", id, id_size);
+	if (result != ASN1_SUCCESS) {
+		gnutls_assert();
+		asn1_delete_structure(&ext);
+		return _gnutls_asn2err(result);
+	}
+
+	asn1_write_value(ext, "authorityCertIssuer", NULL, 0);
+	asn1_write_value(ext, "authorityCertSerialNumber", NULL, 0);
+
+	result = _gnutls_x509_der_encode( ext, "", der_ext, 0);
+
+	asn1_delete_structure(&ext);
+
+	if (result < 0) {
+		gnutls_assert();
+		return result;
+	}
+
+	return 0;
+}
+
+
 /* Creates and encodes the CRL Distribution points. data_string should be a name
  * and type holds the type of the name. 
  * reason_flags should be an or'ed sequence of GNUTLS_CRL_REASON_*.
@@ -687,6 +775,8 @@ int _gnutls_x509_ext_gen_crl_dist_points(gnutls_x509_subject_alt_name type,
 	}
 #endif
 
+	asn1_write_value( ext, "?LAST.distributionPoint.nameRelativeToCRLIssuer", NULL, 0);
+
 	result = write_new_general_name(ext, "?LAST.distributionPoint.fullName", type, data_string);
 	if (result < 0) {
 		gnutls_assert();
