@@ -32,6 +32,8 @@
 #include "certtool-gaa.h"
 #include <gnutls/pkcs12.h>
 #include <unistd.h>
+#include <getpass.h>
+#include <certtool-cfg.h>
 
 static void print_crl_info( gnutls_x509_crl crl, FILE* out, int all);
 int generate_prime(int bits);
@@ -57,8 +59,12 @@ gnutls_x509_crt* load_cert_list(int mand, int *size);
 static gaainfo info;
 FILE* outfile;
 FILE* infile;
-int in_cert_format;
-int out_cert_format;
+static int in_cert_format;
+static int out_cert_format;
+
+/* non interactive operation if set
+ */
+static int batch;
 
 unsigned char buffer[50*1024];
 const int buffer_size = sizeof(buffer);
@@ -138,33 +144,6 @@ int len;
 	return input;
 }
 
-static const char* read_pass( const char* input_str)
-{
-#ifndef HAVE_GETPASS
-static char input[128];
-#endif
-const char* pass;
-
-	if (info.pass) return info.pass;
-
-#ifndef HAVE_GETPASS
-
-	fputs( input_str, stderr);
-	fgets( input, sizeof(input), stdin);
-	
-	input[strlen(input)-1] = 0;
-
-	if (strlen(input)==0 || input[0]=='\n') return NULL;
-
-	return input;
-#else
-	pass = getpass(input_str);
-	if (pass == NULL || strlen(pass)==0 || pass[0]=='\n') return NULL;
-
-	return pass;
-#endif
-}
-
 static int read_yesno( const char* input_str)
 {
 char input[128];
@@ -241,6 +220,470 @@ static void print_key_usage( unsigned int x, FILE* out)
 		fprintf(out,"\t\tKey decipher only.\n");
 }
 
+static void print_key_purpose( const char* x, FILE* out) 
+{
+	if (strcasecmp( x, GNUTLS_KP_TLS_WWW_SERVER)==0)
+		fprintf(out,"\t\tTLS WWW Server.\n");
+	else if (strcasecmp( x, GNUTLS_KP_TLS_WWW_CLIENT)==0)
+		fprintf(out,"\t\tTLS WWW Client.\n");
+	else if (strcasecmp( x, GNUTLS_KP_CODE_SIGNING)==0)
+		fprintf(out,"\t\tCode signing.\n");
+	else if (strcasecmp( x, GNUTLS_KP_EMAIL_PROTECTION)==0)
+		fprintf(out,"\t\tEmail protection.\n");
+	else if (strcasecmp( x, GNUTLS_KP_TIME_STAMPING)==0)
+		fprintf(out,"\t\tTime stamping.\n");
+	else if (strcasecmp( x, GNUTLS_KP_OCSP_SIGNING)==0)
+		fprintf(out,"\t\tOCSP signing.\n");
+	else if (strcasecmp( x, GNUTLS_KP_ANY)==0)
+		fprintf(out,"\t\tAny purpose.\n");
+	else fprintf(out,"\t\t%s\n", x);
+}
+
+
+/* Wrapper functions for non-interactive mode.
+ */
+const char* get_pass(void)
+{
+	if (batch)
+		return password;
+	else
+		return read_pass("Enter password: ");
+}
+
+const char* get_challenge_pass(void)
+{
+	if (batch)
+		return challenge_password;
+	else
+		return read_pass("Enter a challenge password: ");
+}
+
+const char* get_crl_dist_point_url(void)
+{
+	if (batch)
+		return crl_dist_points;
+	else
+		return read_str( "Enter the URI of the CRL distribution point: ");
+}
+
+void get_country_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!country) return;
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_X520_COUNTRY_NAME, 0, 
+			country, strlen(country));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "Country name (2 chars): ", GNUTLS_OID_X520_COUNTRY_NAME);
+	}
+
+}
+
+void get_organization_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!organization) return;
+
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_X520_ORGANIZATION_NAME, 0, 
+			organization, strlen(organization));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "Organization name: ", GNUTLS_OID_X520_ORGANIZATION_NAME);
+	}
+
+}
+
+void get_unit_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!unit) return;
+		
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0, 
+			unit, strlen(unit));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "Organizational unit name: ", GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME);
+	}
+
+}
+
+void get_state_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!state) return;
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0, 
+			state, strlen(state));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "State or province name: ", GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME);
+	}
+
+}
+
+void get_locality_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!locality) return;
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_X520_LOCALITY_NAME, 0, 
+			locality, strlen(locality));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "Locality name: ", GNUTLS_OID_X520_LOCALITY_NAME);
+	}
+
+}
+
+void get_cn_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!cn) return;
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_X520_COMMON_NAME, 0, 
+			cn, strlen(cn));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "Common name: ", GNUTLS_OID_X520_COMMON_NAME);
+	}
+
+}
+
+void get_pkcs9_email_crt_set( gnutls_x509_crt crt)
+{
+int ret;
+
+	if (batch) {
+		if (!pkcs9_email) return;
+		ret = gnutls_x509_crt_set_dn_by_oid(crt, GNUTLS_OID_PKCS9_EMAIL, 0, 
+			pkcs9_email, strlen(pkcs9_email));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crt_set( crt, "E-mail: ", GNUTLS_OID_PKCS9_EMAIL);
+	}
+
+}
+
+int get_serial( void) 
+{
+	if (batch) {
+		if (!serial) return 0;
+		return serial;
+	} else {
+		return read_int( "Enter the certificate's serial number (decimal): ");
+	}
+}
+
+int get_days( void)
+{
+int days;
+
+	if (batch) {
+		if (expiration_days <= 0) return 365;
+		else return expiration_days;
+	} else {
+		do {
+			days = read_int( "The certificate will expire in (days): ");
+		} while( days==0);
+		return days;
+	}
+}
+
+int get_ca_status( void)
+{
+	if (batch) {
+		return ca;
+	} else {
+		return read_yesno( "Does the certificate belong to an authority? (Y/N): ");
+	}
+}
+
+const char* get_pkcs12_key_name( void)
+{
+const char* name;
+
+	if (batch) {
+		if (!pkcs12_key_name) return "Anonymous";
+		return pkcs12_key_name;
+	} else {
+		do {
+			name = read_str("Enter a name for the key: ");
+		} while( name == NULL);
+	}
+	return name;
+}
+
+int get_tls_client_status( void)
+{
+	if (batch) {
+		return tls_www_client;
+	} else {
+		return read_yesno( "Is this a TLS web client certificate? (Y/N): ");
+	}
+}
+
+int get_tls_server_status( void)
+{
+	if (batch) {
+		return tls_www_server;
+	} else {
+		return read_yesno( "Is this also a TLS web server certificate? (Y/N): ");
+	}
+}
+
+const char* get_dns_name( void)
+{
+	if (batch) {
+		return dns_name;
+	} else {
+		return read_str( "Enter the dnsName of the subject of the certificate: ");
+	}
+}
+
+const char* get_email( void)
+{
+	if (batch) {
+		return email;
+	} else {
+		return read_str( "Enter the e-mail of the subject of the certificate: ");
+	}
+}
+
+int get_sign_status( int server)
+{
+const char* msg;
+
+	if (batch) {
+		return signing_key;
+	} else {
+		if (server) msg = "Will the certificate be used for signing (DHE and RSA-EXPORT ciphersuites)? (Y/N): ";
+		else msg = "Will the certificate be used for signing (required for TLS)? (Y/N): ";
+		return read_yesno( msg);
+	}
+}
+
+int get_encrypt_status( int server)
+{
+const char* msg;
+
+	if (batch) {
+		return encryption_key;
+	} else {
+		if (server) msg = "Will the certificate be used for encryption (RSA ciphersuites)? (Y/N): ";
+		else msg = "Will the certificate be used for encryption (not required for TLS)? (Y/N): ";
+		return read_yesno( msg);
+	}
+}
+
+int get_cert_sign_status(void)
+{
+	if (batch) {
+		return cert_sign_key;
+	} else {
+		return read_yesno( "Will the certificate be used to sign other certificates? (Y/N): ");
+	}
+}
+
+int get_crl_sign_status(void)
+{
+	if (batch) {
+		return crl_sign_key;
+	} else {
+		return read_yesno( "Will the certificate be used to sign CRLs? (Y/N): ");
+	}
+}
+
+int get_code_sign_status(void)
+{
+	if (batch) {
+		return code_sign_key;
+	} else {
+		return read_yesno( "Will the certificate be used to sign code? (Y/N): ");
+	}
+}
+
+int get_ocsp_sign_status(void)
+{
+	if (batch) {
+		return ocsp_sign_key;
+	} else {
+		return read_yesno( "Will the certificate be used to sign OCSP requests? (Y/N): ");
+	}
+}
+
+int get_time_stamp_status( void)
+{
+	if (batch) {
+		return time_stamping_key;
+	} else {
+		return read_yesno( "Will the certificate be used for time stamping? (Y/N): ");
+	}
+}
+
+int get_crl_next_update( void)
+{
+int days;
+
+	if (batch) {
+		if (crl_next_update <= 0) return 365;
+		else return crl_next_update;
+	} else {
+		do {
+			days = read_int( "The next CRL will be issued in (days): ");
+		} while( days==0);
+		return days;
+	}
+}
+
+/* CRQ stuff.
+ */
+void get_country_crq_set( gnutls_x509_crq crq)
+{
+int ret;
+
+	if (batch) {
+		if (!country) return;
+		ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_COUNTRY_NAME, 0, 
+			country, strlen(country));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crq_set( crq, "Country name (2 chars): ", GNUTLS_OID_X520_COUNTRY_NAME);
+	}
+
+}
+
+void get_organization_crq_set( gnutls_x509_crq crq)
+{
+int ret;
+
+	if (batch) {
+		if (!organization) return;
+
+		ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_ORGANIZATION_NAME, 0, 
+			organization, strlen(organization));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crq_set( crq, "Organization name: ", GNUTLS_OID_X520_ORGANIZATION_NAME);
+	}
+
+}
+
+void get_unit_crq_set( gnutls_x509_crq crq)
+{
+int ret;
+
+	if (batch) {
+		if (!unit) return;
+		
+		ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0, 
+			unit, strlen(unit));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crq_set( crq, "Organizational unit name: ", GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME);
+	}
+
+}
+
+void get_state_crq_set( gnutls_x509_crq crq)
+{
+int ret;
+
+	if (batch) {
+		if (!state) return;
+		ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0, 
+			state, strlen(state));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crq_set( crq, "State or province name: ", GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME);
+	}
+
+}
+
+void get_locality_crq_set( gnutls_x509_crq crq)
+{
+int ret;
+
+	if (batch) {
+		if (!locality) return;
+		ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_LOCALITY_NAME, 0, 
+			locality, strlen(locality));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crq_set( crq, "Locality name: ", GNUTLS_OID_X520_LOCALITY_NAME);
+	}
+
+}
+
+void get_cn_crq_set( gnutls_x509_crq crq)
+{
+int ret;
+
+	if (batch) {
+		if (!cn) return;
+		ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_COMMON_NAME, 0, 
+			cn, strlen(cn));
+		if (ret < 0) {
+			fprintf(stderr, "set_dn: %s\n", gnutls_strerror(ret));
+			exit(1);
+		}
+	} else {
+		read_crq_set( crq, "Common name: ", GNUTLS_OID_X520_COMMON_NAME);
+	}
+
+}
+
+
+
+
+
+
+
+
 static void print_private_key( gnutls_x509_privkey key)
 {
 int ret;
@@ -261,7 +704,7 @@ size_t size;
 		
 		if (info.export) flags = GNUTLS_PKCS_USE_PKCS12_RC2_40;
 		else flags = GNUTLS_PKCS_USE_PKCS12_3DES;
-		if ((pass=read_pass("Enter password: ")) == NULL) flags = GNUTLS_PKCS_PLAIN;
+		if ((pass=get_pass()) == NULL) flags = GNUTLS_PKCS_PLAIN;
 
 		size = sizeof(buffer);
 		ret = gnutls_x509_privkey_export_pkcs8( key, out_cert_format, pass, flags, buffer, &size);
@@ -288,11 +731,12 @@ gnutls_x509_privkey key;
 }
 
 
-gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
+gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key,
+	gnutls_x509_crt ca_crt)
 {
 	gnutls_x509_crt crt;
 	gnutls_x509_privkey key = NULL;
-	int size, serial;
+	int size, serial, client;
 	int days, result, ca_status;
 	const char* str;
 	int vers = 3; /* the default version in the certificate 
@@ -313,18 +757,20 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 
 		key = load_private_key(1);
 
+		if (!batch)
 		fprintf(stderr, "Please enter the details of the certificate's distinguished name. "
 		"Just press enter to ignore a field.\n");
 
-		read_crt_set( crt, "Country name (2 chars): ", GNUTLS_OID_X520_COUNTRY_NAME);
-		read_crt_set( crt, "Organization name: ", GNUTLS_OID_X520_ORGANIZATION_NAME);
-		read_crt_set( crt, "Organizational unit name: ", GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME);
-		read_crt_set( crt, "Locality name: ", GNUTLS_OID_X520_LOCALITY_NAME);
-		read_crt_set( crt, "State or province name: ", GNUTLS_OID_X520_LOCALITY_NAME);
-		read_crt_set( crt, "Common name: ", GNUTLS_OID_X520_COMMON_NAME);
+		get_country_crt_set( crt);
+		get_organization_crt_set(crt);
+		get_unit_crt_set( crt);
+		get_locality_crt_set( crt);
+		get_state_crt_set( crt);
+		get_cn_crt_set( crt);
 	
-		fprintf(stderr, "This field should not be used in new certificates.\n");
-		read_crt_set( crt, "E-mail: ", GNUTLS_OID_PKCS9_EMAIL);
+		if (!batch) fprintf(stderr, "This field should not be used in new certificates.\n");
+
+		get_pkcs9_email_crt_set( crt);
 
 		result = gnutls_x509_crt_set_key( crt, key);
 		if (result < 0) {
@@ -341,7 +787,7 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 	}
 
 
-	serial = read_int( "Enter the certificate's serial number (decimal): ");
+	serial = get_serial();
 	buffer[3] = serial & 0xff;
 	buffer[2] = (serial >> 8) & 0xff;
 	buffer[1] = (serial >> 16) & 0xff;
@@ -354,12 +800,11 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 	}
 
 
-	fprintf(stderr, "\n\nActivation/Expiration time.\n");	
+	if (!batch) fprintf(stderr, "\n\nActivation/Expiration time.\n");	
+
 	gnutls_x509_crt_set_activation_time( crt, time(NULL));
 
-	do {
-		days = read_int( "The generated certificate will expire in (days): ");
-	} while( days==0);
+	days = get_days();
 	
 	result = gnutls_x509_crt_set_expiration_time( crt, time(NULL)+days*24*60*60);
 	if (result < 0) {
@@ -368,9 +813,9 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 	}
 	
 
-	fprintf(stderr, "\n\nExtensions.\n");	
+	if (!batch) fprintf(stderr, "\n\nExtensions.\n");	
 
-	ca_status = read_yesno( "Does the certificate belong to an authority? (Y/N): ");
+	ca_status = get_ca_status();
 
 	result = gnutls_x509_crt_set_ca_status( crt, ca_status);
 	if (result < 0) {
@@ -378,9 +823,18 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 		exit(1);
 	}
 
-	server = read_yesno( "Is this a web server certificate? (Y/N): ");
+	client = get_tls_client_status();
+	if (client != 0) {
+		result = gnutls_x509_crt_set_key_purpose_oid( crt, GNUTLS_KP_TLS_WWW_CLIENT, 0);
+		if (result < 0) {
+			fprintf(stderr, "key_kp: %s\n", gnutls_strerror(result));
+			exit(1);
+		}
+	}
+
+	server = get_tls_server_status();
 	if (server != 0) {
-		str = read_str( "Enter the dnsName of the subject of the certificate: ");
+		str = get_dns_name();
 		if (str != NULL) {
 			result = gnutls_x509_crt_set_subject_alternative_name( crt, GNUTLS_SAN_DNSNAME, str);
 			if (result < 0) {
@@ -388,9 +842,16 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 				exit(1);
 			}
 		}
+
+		result = gnutls_x509_crt_set_key_purpose_oid( crt, GNUTLS_KP_TLS_WWW_SERVER, 0);
+		if (result < 0) {
+			fprintf(stderr, "key_kp: %s\n", gnutls_strerror(result));
+			exit(1);
+		}
+		
 	} else {
 
-		str = read_str( "Enter the e-mail of the subject of the certificate: ");
+		str = get_email();
 	
 		if (str != NULL) {
 			result = gnutls_x509_crt_set_subject_alternative_name( crt, GNUTLS_SAN_RFC822NAME, str);
@@ -404,33 +865,54 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 
 	if (!ca_status || server) {
 		int pk;
-		const char* msg1, *msg2;
-		
-		if (server) msg1 = "Will the certificate be used for signing (DHE and RSA-EXPORT ciphersuites)? (Y/N): ";
-		else msg1 = "Will the certificate be used for signing (required for TLS)? (Y/N): ";
 
-		if (server) msg2 = "Will the certificate be used for encryption (RSA ciphersuites)? (Y/N): ";
-		else msg2 = "Will the certificate be used for encryption (not required for TLS)? (Y/N): ";
 
 		pk = gnutls_x509_crt_get_pk_algorithm( crt, NULL);
 
 		if (pk != GNUTLS_PK_DSA) { /* DSA keys can only sign.
 				  	    */
-			result = read_yesno( msg1);
+			result = get_sign_status(server);
 			if (result) usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
 
-			result = read_yesno( msg2);
+			result = get_encrypt_status( server);
 			if (result) usage |= GNUTLS_KEY_KEY_ENCIPHERMENT;
 		} else usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
 	}
 
 
 	if (ca_status) {
-		result = read_yesno( "Will the certificate be used to sign other certificates? (Y/N): ");
+		result = get_cert_sign_status();
 		if (result) usage |= GNUTLS_KEY_KEY_CERT_SIGN;
 
-		result = read_yesno( "Will the certificate be used to sign CRLs? (Y/N): ");
+		result = get_crl_sign_status();
 		if (result) usage |= GNUTLS_KEY_CRL_SIGN;
+
+		result = get_code_sign_status();
+		if (result) {
+			result = gnutls_x509_crt_set_key_purpose_oid( crt, GNUTLS_KP_CODE_SIGNING, 0);
+			if (result < 0) {
+				fprintf(stderr, "key_kp: %s\n", gnutls_strerror(result));
+				exit(1);
+			}
+		}
+
+		result = get_ocsp_sign_status();
+		if (result) {
+			result = gnutls_x509_crt_set_key_purpose_oid( crt, GNUTLS_KP_OCSP_SIGNING, 0);
+			if (result < 0) {
+				fprintf(stderr, "key_kp: %s\n", gnutls_strerror(result));
+				exit(1);
+			}
+		}
+
+		result = get_time_stamp_status();
+		if (result) {
+			result = gnutls_x509_crt_set_key_purpose_oid( crt, GNUTLS_KP_TIME_STAMPING, 0);
+			if (result < 0) {
+				fprintf(stderr, "key_kp: %s\n", gnutls_strerror(result));
+				exit(1);
+			}
+		}
 	}
 
 	if (usage != 0) {
@@ -461,6 +943,20 @@ gnutls_x509_crt generate_certificate( gnutls_x509_privkey *ret_key)
 		}
 	}
 
+	/* Authority Key ID.
+	 */
+	if (ca_crt != NULL) {
+		size = sizeof(buffer);
+		result = gnutls_x509_crt_get_key_id(ca_crt, 0, buffer, &size);
+		if (result >= 0) {
+			result = gnutls_x509_crt_set_authority_key_id( crt, buffer, size);
+			if (result < 0) {
+				fprintf(stderr, "set_authority_key_id: %s\n", gnutls_strerror(result));
+				exit(1);
+			}
+		}
+	}
+
 	*ret_key = key;
 	return crt;
 
@@ -495,9 +991,7 @@ gnutls_x509_crl generate_crl( void)
 	fprintf(stderr, "\n\nthisUpdate/nextUpdate time.\n");	
 	gnutls_x509_crl_set_this_update( crl, time(NULL));
 
-	do {
-		days = read_int( "The next CRL will be issued in (days): ");
-	} while( days==0);
+	days = get_crl_next_update();
 	
 	result = gnutls_x509_crl_set_next_update( crl, time(NULL)+days*24*60*60);
 	if (result < 0) {
@@ -533,9 +1027,7 @@ gnutls_x509_crt update_certificate( void)
 	fprintf(stderr, "Activation/Expiration time.\n");	
 	gnutls_x509_crt_set_activation_time( crt, time(NULL));
 	
-	do {
-		days = read_int( "The updated certificate will expire in (days): ");
-	} while( days==0);
+	days = get_days();
 	
 	result = gnutls_x509_crt_set_expiration_time( crt, time(NULL)+days*24*60*60);
 	if (result < 0) {
@@ -558,9 +1050,9 @@ void generate_self_signed( void)
 
 	fprintf(stderr, "Generating a self signed certificate...\n");
 
-	crt = generate_certificate( &key);
+	crt = generate_certificate( &key, NULL);
 
-	uri = read_str( "Enter the URI of the CRL distribution point: ");
+	uri = get_crl_dist_point_url();
 	if (uri) {
 		result = gnutls_x509_crt_set_crl_dist_points( crt, GNUTLS_SAN_URI,
 			uri, 0 /* all reasons */);
@@ -607,7 +1099,7 @@ void generate_signed_certificate( void)
 	ca_key = load_ca_private_key();
 	ca_crt = load_ca_cert();
 
-	crt = generate_certificate( &key);
+	crt = generate_certificate( &key, ca_crt);
 	
 	/* Copy the CRL distribution points.
 	 */
@@ -742,14 +1234,20 @@ int ret;
 	if (info.outcert_format) out_cert_format = GNUTLS_X509_FMT_DER;
 	else out_cert_format = GNUTLS_X509_FMT_PEM;
 
-	if ((ret=gnutls_global_init()) < 0) {
-		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
-		exit(1);
+	batch = 0;
+	if (info.template) {
+		batch = 1;
+		parse_template( info.template);
 	}
 
 	gnutls_global_set_log_function( tls_log_func);
 	gnutls_global_set_log_level(info.debug);
 
+	if ((ret=gnutls_global_init()) < 0) {
+		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
+		exit(1);
+	}
+
 	switch( info.action) {
 		case 0:
 			generate_self_signed();
@@ -842,7 +1340,9 @@ static inline int known_oid( const char* oid)
 	if (strcmp(oid, "2.5.29.17") == 0 ||
 		strcmp( oid, "2.5.29.19") == 0 ||
 		strcmp( oid, "2.5.29.31") == 0 ||
+		strcmp( oid, "2.5.29.37") == 0 ||
 		strcmp( oid, "2.5.29.14") == 0 ||
+		strcmp( oid, "2.5.29.35") == 0 ||
 		strcmp( oid, "2.5.29.15") == 0)
 			return 1;
 	
@@ -1049,6 +1549,24 @@ static void print_certificate_info( gnutls_x509_crt crt, FILE* out, unsigned int
 		print_key_usage(key_usage, out);
 	}
 
+	/* Key Purpose identifiers.
+	 */
+	i = 0;
+	
+	size = sizeof(buffer);
+	if ((ret=gnutls_x509_crt_get_key_purpose_oid( crt, 0, buffer, &size, &critical)) >= 0) {
+		fprintf(out, "\tKey purpose OIDs: %s\n", critical?"(critical)":"");
+		do {
+			size = sizeof(buffer);
+			ret = gnutls_x509_crt_get_key_purpose_oid( crt, i, buffer, &size, &critical);
+	
+			if (ret >= 0) {
+				print_key_purpose(buffer, out);
+			}
+			i++;
+		} while(ret >= 0);
+	}
+
 	/* Subject Key ID 
 	 */
 	size = sizeof(buffer);
@@ -1068,6 +1586,25 @@ static void print_certificate_info( gnutls_x509_crt crt, FILE* out, unsigned int
 		fprintf(out, "\tSubject Key ID: %s\n\t\t%s\n", critical?"(critical)":"", printable);
 	}
 
+	/* Authority Key ID 
+	 */
+	size = sizeof(buffer);
+	ret = gnutls_x509_crt_get_authority_key_id(crt, buffer, &size, &critical);
+
+	if (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
+	{
+	    	fprintf(out, "Error getting authority key id: %s\n", gnutls_strerror(ret));
+	}
+	
+	if (ret >= 0) {
+		print = printable;
+		for (i = 0; i < size; i++) {
+			sprintf(print, "%.2x ", (unsigned char) buffer[i]);
+			print += 3;
+		}
+		fprintf(out, "\tAuthority Key ID: %s\n\t\t%s\n", critical?"(critical)":"", printable);
+	}
+
 	/* other extensions:
 	 */
 	indx = 0;
@@ -1145,7 +1682,7 @@ static void print_certificate_info( gnutls_x509_crt crt, FILE* out, unsigned int
 
 	fprintf(out, "\n");
 
-	if (out==stderr) /* interactive */
+	if (out==stderr && batch == 0) /* interactive */
 	if (read_yesno( "Is the above information ok? (Y/N): ")==0) {
 		exit(1);
 	}
@@ -1279,7 +1816,7 @@ void privkey_info( void)
 	if (!info.pkcs8) {
 		ret = gnutls_x509_privkey_import(key, &pem, in_cert_format);
 	} else {
-		pass = read_pass("Enter password: ");
+		pass = get_pass();
 		ret = gnutls_x509_privkey_import_pkcs8(key, &pem, in_cert_format, pass, 0);
 	}
 
@@ -1361,7 +1898,7 @@ const char* pass;
 	if (!info.pkcs8)
 		ret = gnutls_x509_privkey_import( key, &dat, in_cert_format);
 	else {
-		pass = read_pass("Enter password: ");
+		pass = get_pass();
 		ret = gnutls_x509_privkey_import_pkcs8( key, &dat, in_cert_format,
 			pass, 0);
 	}
@@ -1453,7 +1990,7 @@ size_t size;
 	if (!info.pkcs8)
 		ret = gnutls_x509_privkey_import( key, &dat, in_cert_format);
 	else {
-		pass = read_pass("Enter password: ");
+		pass = get_pass();
 		ret = gnutls_x509_privkey_import_pkcs8( key, &dat, in_cert_format,
 			pass, 0);
 	}
@@ -1619,12 +2156,12 @@ void generate_request(void)
 	 */
 	key = generate_private_key_int();
 
-	read_crq_set( crq, "Country name (2 chars): ", GNUTLS_OID_X520_COUNTRY_NAME);
-	read_crq_set( crq, "Organization name: ", GNUTLS_OID_X520_ORGANIZATION_NAME);
-	read_crq_set( crq, "Organizational unit name: ", GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME);
-	read_crq_set( crq, "Locality name: ", GNUTLS_OID_X520_LOCALITY_NAME);
-	read_crq_set( crq, "State or province name: ", GNUTLS_OID_X520_LOCALITY_NAME);
-	read_crq_set( crq, "Common name: ", GNUTLS_OID_X520_COMMON_NAME);
+	get_country_crq_set( crq);
+	get_organization_crq_set(crq);
+	get_unit_crq_set( crq);
+	get_locality_crq_set( crq);
+	get_state_crq_set( crq);
+	get_cn_crq_set( crq);
 
 	ret = gnutls_x509_crq_set_version( crq, 1);
 	if (ret < 0) {
@@ -1632,7 +2169,7 @@ void generate_request(void)
 		exit(1);
 	}
 
-	pass = read_pass("Enter a challenge password: ");
+	pass = get_challenge_pass();
 	
 	if (pass != NULL) {
 		ret = gnutls_x509_crq_set_challenge_password( crq, pass);
@@ -2051,11 +2588,9 @@ void generate_pkcs12( void)
 	key = load_private_key(1);
 	crt = load_cert(0);
 	
-	do {
-		name = read_str("Enter a name for the key: ");
-	} while( name == NULL);
+	name = get_pkcs12_key_name();
 
-	password = read_pass( "Enter password: ");
+	password = get_pass();
 
 	result = gnutls_pkcs12_bag_init( &bag);
 	if (result < 0) {
@@ -2288,7 +2823,7 @@ void pkcs12_info( void)
 	data.data = buffer;
 	data.size = size;
 
-	password = read_pass( "Enter password: ");
+	password = get_pass();
 	
 	result = gnutls_pkcs12_init(&pkcs12);
 	if (result < 0) {
